"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.unit.rules import BaseRuleTestCase


class TestPropertyOnlyOne(BaseRuleTestCase):
    """Test OnlyOne Property Configuration"""
