[
    {
        "op": "test",
        "path": "/createOnlyProperties",
        "value": [
            "/properties/Tags",
            "/properties/ExperimentOptions/properties/AccountTargeting",
        ],
    },
    {
        "op": "replace",
        "path": "/createOnlyProperties",
        "value": ["/properties/Tags", "/properties/ExperimentOptions/AccountTargeting"],
    },
]
